{-------------------------------------------------------------}
{
{           cartaVox - leitor Mime
{
{-------------------------------------------------------------}

unit carLeit;

interface

uses
    dvcrt,
    dvWin,
    dvHora,
    dvform,
    sysutils,
    windows,
    carCopia,
    carEst,
    carDecod,
    carMsg,
    carTela,
    carUtil,
    carVars,
    carSMTP,
    uuenc,
    dvexec;

function gravaParteDecodificada (parte: PEstrutura; nomeArq: string): string;
procedure editaArquivo (nomeArq: string);
procedure leCarta (ncar: integer; lerSempre: boolean);

implementation

var
    tipoGravacaoSimplificada: boolean;
    totalPartesFolhe: integer;
    gravacaoForcada: boolean;

{-------------------------------------------------------------}
{       Grava o arquivo da carta sem os anexos apagados.
{-------------------------------------------------------------}

    procedure  checaPaiSemFilhos (umPai: PEstrutura);
    var
        tenhoFilhos: boolean;
        i: integer;
        parte: PEstrutura;
    begin
        tenhoFilhos := false;
        if umPai^.carta <> NIL then
            for i := 0 to (umPai^.carta^.partescarta.count-1) do
                begin
                    parte := umPai^.carta^.partescarta[i];
                    if not parte^.removido then
                        begin
                            tenhoFilhos := true;
                            break
                        end;
                end;

        if not tenhoFilhos then
            begin
                umPai^.removido := true;
                if umPai^.ptPai <> NIL then
                    checaPaiSemFilhos (umpai^.ptPai);
            end;
    end;

    {---------------------------------------------------------}

    procedure selecionarRemovidos;
    var
        i: integer;
        parte: PEstrutura;
    begin
        for i := 0 to (listaDePartes.count-1) do
            begin
                parte := listaDePartes[i];
                if parte^.removido then
                    checaPaiSemFilhos (parte^.ptPai);
            end;
    end;

    {---------------------------------------------------------}

function gravarAlteracoesNoArquivo (nomeArq: string; nCar: integer): boolean;
var
    i: integer;
    parte: PEstrutura;
    arq: text;
    houveErro: boolean;
    boundary: string;

    {---------------------------------------------------------}

    procedure escreveNoArquivo (lIni, lFim: integer);
    var
        s: string;
    begin
        while (not houveErro) and (lIni <= lFim) and
             (lIni >= 0) and (lIni < linhasArquivo.count) do   // programao defensiva
            begin
                s := linhasArquivo[LIni];
                {$I-} writeln (arq, s); {$I+}
                if ioresult <> 0 then
                    begin
                        msgBaixo ('CTERRDSK');   {'Erro de escrita no disco'}
                        houveErro := true;
                    end;
                lIni := lIni + 1;
            end;
    end;

    {---------------------------------------------------------}

    procedure gravarOriginal (aCarta: PEstrutura);
    var
        i: integer;
        umaParte: PEstrutura;
    begin
        if aCarta^.removido then exit;
        if aCarta^.boundary <> '' then
            escreveNoArquivo (aCarta^.linhaInicialCab, aCarta^.linhaInicial - 2)
        else
            escreveNoArquivo (aCarta^.linhaInicialCab, aCarta^.linhaInicial - 1);

        for i := 0 to (aCarta^.carta^.partescarta.count - 1) do
            begin
                umaParte := aCarta^.carta^.partescarta[i];
                if umaParte^.removido then continue;
                escreveNoArquivo (umaParte^.linhaInicialCab-1, umaParte^.linhaInicial-1);
                if umaParte^.parte^.subCarta <> NIL then
                    begin
                        gravarOriginal (umaParte^.parte^.subCarta);
                    end
                else
                    escreveNoArquivo (umaParte^.linhaInicial, umaParte^.linhaFinal);
            end;

        if aCarta^.boundary <> '' then
            escreveNoArquivo (aCarta^.linhaFinal+1, aCarta^.linhaFinal+1);
    end;


begin
    gravarAlteracoesNoArquivo := false;
    assign (arq, nomeArq);
    {$I-}  rewrite (arq);   {$I+}
    if ioresult <> 0 then
        begin
            msgBaixo ('CTERRDSK'); {'Erro de escrita no disco'}
            exit;
        end;

    houveErro := false;
    if tipoGravacaoSimplificada then
        begin
            boundary := regLido[ncar]^.boundary;
            escreveNoArquivo (0, regLido [nCar]^.linhaInicial - 2);
            for i := 0 to (listaDePartes.count - 1) do
                begin
                    parte := listaDePartes[i];
                    if not parte^.removido then
                        begin
                            {$I-} writeln (arq, '--'+boundary); {$I+}
                            if ioresult <> 0 then
                                begin
                                    msgBaixo ('CTERRDSK');   {'Erro de escrita no disco'}
                                    houveErro := true;
                                    break;
                                end;
                            escreveNoArquivo (parte^.linhaInicialCab, parte^.linhaFinal);
                        end;
                end;

            {$I-} writeln (arq, '--'+ boundary + '--'); {$I+}
            if ioresult <> 0 then;
        end
    else
        begin
            selecionarRemovidos;
            gravarOriginal (regLido[nCar]);
        end;

    if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
        begin
            {$I-} writeln (arq, '.'); {$I+}
            if ioresult <> 0 then;
        end;
    {$I-} close (arq); {$I+}
    if (ioresult = 0) and (not houveErro) then
        begin
            msgBaixo ('CTARQGRA'); {'Arquivo gravado'}
            gravarAlteracoesNoArquivo := true;
        end;
end;

{-------------------------------------------------------------}
{       Regrava o arquivo sem as partes apagadas
{-------------------------------------------------------------}

procedure gravarAlteracoes (nCar: integer);
var
    c, c2: char;
    nomeArq: string;

label outroNomeArq, desistiu;
begin
    if not gravacaoForcada then
       if (totalPartesFolhe = listaDePartes.count) or
          (totalPartesFolhe < 1) then exit;

    repeat
        limpaParteTela (20, 25);
        textBackground (RED);
        mensagem ('CTGRAALT', 0); {'Deseja gravar as alteraes na carta?'}
        sintLeTecla (c, c2);
        textBackground (BLACK);
        writeln;
    until upcase (c) in ['S', 'N', ENTER, ESC];
    if upcase (c) in ['N', ESC] then goto desistiu;

    repeat
        limpaParteTela (22, 25);
        textBackground (RED);
        mensagem ('CTGRAVSO', 0); {'Qual o tipo de gravao, Simplificada ou Original?'}
        sintLeTecla (c, c2);
        textBackground (BLACK);
        writeln;
    until upcase (c) in ['S', 'O', ENTER, ESC];
    if c = ESC then goto desistiu;

    tipoGravacaoSimplificada := upcase (c) <> 'O';

outroNomeArq:
    limpaParteTela (24, 25);
    textBackground (RED);
    mensagem ('CTEDENSO', 0); {'Edite o nome do arquivo ou tecle ENTER para sobrescrever'}
    textBackground (BLACK);
    writeln;
    nomeArq := regLido [nCar]^.carta^.nomArqCarta;
    sintetiza (nomeArq);
    c := sintEditaCampo (nomeArq, 1, wherey, 255, 80, true);
    if (c = ESC) or (trim (nomeArq) = '') or
       (pos ('.', nomeArq) = 0) then
        msgBaixo ('CTDESIST') {'Desistiu'}
    else
    if (trim (nomeArq) <> trim (regLido [nCar]^.carta^.nomArqCarta)) and
       existeArq (nomeArq) then
        begin
            repeat
                limpaParteTela (24, 25);
                textBackground (RED);
                mensagem ('CTJAEXI1', 0); {'O arquivo destino'}
                writeln (' ');
                delay (100);
                sintWrite (nomeArq);
                delay (100);
                mensagem ('CTJAEXI2', 0); {'j existe.  Sobrescreve (S/N/ESC)? '}
                c := upcase (sintReadkey);
                write (c);
                textBackground (BLACK);
                writeln;
            until c in ['S', 'N', ENTER, ESC];

            if c = 'N' then
                goTo outroNomeArq
            else
            if c = ESC then
                goto desistiu
            else
                gravarAlteracoesNoArquivo (nomeArq, nCar);
        end
    else
        gravarAlteracoesNoArquivo (nomeArq, nCar);

//neno ERRO    regLido [numRegs]^.carta^.tamanho := filesize (regLido [numRegs]^.carta^.nomArqCarta);
    exit;
desistiu:
    msgBaixo ('CTDESIST') {'Desistiu'}
end;

{-------------------------------------------------------------}
{       Edita o arquivo
{-------------------------------------------------------------}

procedure editaArquivo (nomeArq: string);
var
    nomeProg, nomeDir: string;
begin
    while sintFalando do waitMessage;
    nomeProg := sintAmbiente ('CARTAVOX', 'EDITOR');

    if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
        nomeDir := dirEnvia
    else
        nomeDir := dirRecebe;

    nomeArq := trim (nomeArq);
    if pos (' ', nomeArq) <> 0 then
        if nomeArq [1] <> '"' then
            nomeArq := '"' + nomeArq + '"';
    if executaProg (nomeProg, nomeDir, nomeArq) >= 32 then;
    esperaProgVoltar;
    while sintFalando do waitMessage;
end;

{-------------------------------------------------------------}
{       Fala o assunto da carta.
{-------------------------------------------------------------}

procedure falaAssunto (assunto: string);
begin
    mensagem ('CTASSUNT', -1);  {'Assunto: '}
    assunto := pegaPrefixoAssunto(assunto) + limpaAssunto(assunto);
    sintetiza (assunto);
end;

{-------------------------------------------------------------}
{       Fala o tipo da parte
{-------------------------------------------------------------}

procedure falaTipoDaParte (nParte: integer);
var
    s: string;
begin
    s := typeOuContent_Type (listaDePartes[nParte]);
    if trim (s) = '' then s := pegaTextoMensagem ('CTNAODEF'); {'No definido'}
    sintetiza (s);
    delay (200);
end;

{-------------------------------------------------------------}
{       Fala o nome do arquivo da parte.
{-------------------------------------------------------------}

function pegaNomeDoArquivoDaParte (parte: PEstrutura): string;
begin
    if parte^.carta <> NIL then
        pegaNomeDoArquivoDaParte := parte^.carta^.fileName
    else
        pegaNomeDoArquivoDaParte := parte^.parte^.filename;
end;

procedure falaNomeDoArquivoDaParte (nParte: integer);
var
    parte: PEstrutura;
    arq: string;
begin
    parte := listaDePartes[nParte];
    if parte^.carta <> NIL then
        arq := parte^.carta^.nomArqCarta
    else
        arq := parte^.parte^.filename;
    if trim (arq) = '' then
        arq := pegaTextoMensagem ('CTNAODEF'); {'No definido'}

    sintetiza (arq);
    delay (200);
end;

{-------------------------------------------------------------}
{       Apaga uma parte ou as selecionadas
{-------------------------------------------------------------}

procedure apagaParte (nFolhe: integer);
var
    c: char;
    i: integer;
    item: string;
    selecionado: boolean;
    parte: PEstrutura;

    procedure apagaUmaParte (i: integer);
    begin
        folheiaObtemItem (i, item, selecionado);
        repeat
            msgBaixo ('CTDEAPPA'); {'Deseja apagar essa parte? '}
            sintetiza (copy (item, 6, 999));
            c := upcase (sintReadkey);
        until c in ['S', 'N', ENTER, ESC];

        if c in ['N', ESC] then
            msgBaixo ('CTDESIST') {'Desistiu'}
        else
            begin
                folheiaRemoveItem (i);
                totalPartesFolhe := folheiaNumItens;
                Parte := listaDePartes[strToInt(trim(copy(item, 1, 5)))-1];
                parte^.removido := TRUE;
                msgBaixo ('CTPARAPA'); {'Parte apagada'}
            end;
    end;

begin
    if temItemSelecionado then
        begin
            repeat
                msgBaixo ('CTDEAPSE'); {'Deseja apagar as partes selecionadas?'}
                c := upcase (sintReadkey);
            until c in ['S', 'N', ENTER, ESC];

            if c = ESC then
                    msgBaixo ('CTDESIST') {'Desistiu'}
            else
            if c = 'N' then
                apagaUmaParte (nFolhe)
            else
                begin
                    i := folheiaNumItens;
                    while i >= 0 do
                        begin
                            folheiaObtemItem (i, item, selecionado);
                            if selecionado then
                                begin
                                    folheiaRemoveItem (i);
                                    totalPartesFolhe := folheiaNumItens;
                                    Parte := listaDePartes[strToInt(trim(copy(item, 1, 5)))-1];
                                    parte^.removido := TRUE;
                                end;

                            i := i - 1;
                        end;
                    msgBaixo ('CTPASAPS'); {'Partes apagadas'}
                end;
        end
    else
        apagaUmaParte (nFolhe)
end;

{-------------------------------------------------------------}
{       Grava a parte no arquivo temporrio j decodificado
{-------------------------------------------------------------}

function gravaParteDecodificada (parte: PEstrutura; nomeArq: string): string;
var
    arqTemp: text;
    tempPath, tempFileName: array [0..144] of char;
    houveErro, usaTempFilename: boolean;
    s: string;
    tipoCodif, tipoCharset: char;
    i: integer;
begin
    gravaParteDecodificada := '';
    if nomeArq = '' then
        begin
            usaTempFileName := TRUE;
            getTempPath (144, tempPath);
            GetTempFileName(tempPath, 'CAR', 0, TempFileName);
            nomeArq := strPas(TempFileName);
        end
    else
        usaTempFileName := false;


    assign (arqTemp, nomeArq);
    {$I-}  rewrite (arqTemp);   {$I+}
    if ioresult <> 0 then
        begin
            msgBaixo ('CTERRTMP');  {'Problemas no arquivo temporrio'}
            exit;
        end;

    s := trim (maiuscAnsi (parte^.content_transfer_encoding));

    if s = 'QUOTED-PRINTABLE' then        tipoCodif := 'Q'
    else if s = 'X-UUENCODE' then         tipoCodif := 'X'
    else if s = 'BASE64' then             tipoCodif := 'B'
    else tipoCodif := 'N';

    s := trim (maiuscAnsi (parte^.charset));
    if copy (s, 1, 3) = 'UTF' then tipoCharset := 'U'
    else if copy (s, 1, 3) = 'ISO' then tipoCharset := 'I'
    else tipoCharset := 'N';

    houveErro := false;
    i := parte^.linhaInicial;
    while (not houveErro) and (i <= parte^.linhaFinal) do
        begin
            s := linhasArquivo[i];
            if (i = parte^.linhaFinal ) and (s = '.') then
break;
            if (trim(s) <> '') and (s [length(s)] = '=') then
                begin
                    if trim(s)[1] = '=' then s := trim(s);
                    case tipoCodif of
                        'Q': s := convQuotedPrintable (s);
                        'B': if usaTempFileName then
                                s := DecodFraseMime64 (s);
                        'N':; //nada faz
                    end;
                    case tipoCharset of
                        'U': s := utfToAnsi (s);
                        'I': decodificarString (s);
                        'N':;//nada faz
                    end;

                    {$I-} write (arqTemp, s); {$I+}
                    if ioresult <> 0 then
                        begin
                            msgBaixo ('CTERRDSK');   {'Erro de escrita no disco'}
                            houveErro := true;
                        end;
                end
            else
                begin
                    case tipoCodif of
                        'Q': s := convQuotedPrintable(s);
                        'B': if usaTempFileName then
                                s := DecodFraseMime64 (s);
                    end;
                    case tipoCharset of
                        'U': s := utfToAnsi (s);
                        'I': decodificarString (s);
                    end;
                    {$I-} writeln (arqTemp, s); {$I+}
                    if ioresult <> 0 then
                        begin
                            msgBaixo ('CTERRDSK');   {'Erro de escrita no disco'}
                            houveErro := true;
                        end;
                end;
            i := i + 1;
        end;

    {$I-} close (arqtemp); {$I+}
    if ioresult = 0 then
        if tipoCodif = 'X' then
            if uudecode (nomeArq) <> 0 then
                     msgBaixo ('CTERRDSK'); {'Erro de escrita no disco'}

    if not houveErro then
        gravaParteDecodificada := nomeArq;
end;

{-------------------------------------------------------------}
{     Testa se o arquivo existe, se existir pede outro nome.
{-------------------------------------------------------------}

function testaRetornaNomeArq (nomeArq, diretorio, tipoDaParte: string): string;
var
    c, c2: char;
    editaNomeArq: boolean;
begin
    if diretorio = '' then editaNomeArq := TRUE
    else editaNomeArq := false;
    c := 'S';
    repeat
        if editaNomeArq then
            begin
                repeat
                    limpaParteTela (20, 25);
                    textBackground (RED);
                    mensagem ('CTVOUGRV', 1);   {'Pretendo gravar '}
                    sintWriteln (nomeArq);
                    mensagem ('CTEDINOM', 0);   {'Editore o nome, tecle ENTER para confirmar
                                                            ou ESC para cancelar: '}
                    textBackground (BLACK);
                    writeln;
                    c := sintEdita (nomeArq, 1, wherey, 255, true);
                    writeln;
                until c in [ENTER, ESC];
            end
        else
        if trim (nomeArq) = '' then {Arquivo sem nome, pede o nome.}
            begin
                repeat
                    limpaParteTela (20, 25);
                    textBackground (RED);
                    mensagem ('CTAPATIP', 0); {'A parte do tipo '}
                    sintWriteln (tipoDaParte);
                    mensagem ('CTNTEMNO', 1); {'No tem nome para o arquivo destino.'}
                    mensagem ('CTINFDST', 0); {'Informe o nome completo do arquivo destino:'}
                    textBackground (BLACK);
                    writeln;
                    c := sintEdita (nomeArq, 1, wherey, 255, true);
                    writeln;
                until upcase (c) in [ENTER, ESC];
            end;
        if (c = ESC) or (trim (nomeArq) = '') then
            begin
                msgBaixo ('CTDESIST'); {'Desistiu'}
                testaRetornaNomeArq := '';
                exit;
            end;

        if (pos ('\', nomeArq) = 0) and (pos ('/', nomeArq) = 0) and
           (diretorio = '') and (dirAnexo <> '') then
            nomeArq := dirAnexo + '\' + nomeArq
        else
        if (pos ('\', nomeArq) = 0) and (pos ('/', nomeArq) = 0) and
           (diretorio <> '') then
            nomeArq := diretorio + '\' + nomeArq;

        if existeArq (nomeArq) then
            begin
                repeat
                    limpaParteTela (20, 25);
                    textBackground (RED);
                    mensagem ('CTJAEXI1', 0); {'O arquivo destino'}
                    writeln (' ');
                    delay (100);
                    sintWrite (nomeArq);
                    delay (100);
                    mensagem ('CTJAEXI2', 0); {'j existe.  Sobrescreve (S/N/T/ESC)? '}
                    sintLeTecla (c, c2);
                    textBackground (BLACK);
                    writeln;
                until upcase (c) in ['S', 'N', ENTER, ESC];
                if upcase (c) in ['S', ENTER] then
                    testaRetornaNomeArq := nomeArq
                else
                if c = ESC then
                    begin
                        msgBaixo ('CTDESIST'); {'Desistiu'}
                        testaRetornaNomeArq := '';
                    end
                else
                    editaNomeArq := TRUE;
            end
        else
            testaRetornaNomeArq := nomeArq;

    until upcase (c) in ['S', ENTER, ESC];
end;

{-------------------------------------------------------------}
{       Joga a parte da carta no editor ou no brouse, arquivo temporrio.
{-------------------------------------------------------------}

procedure LeituraParteCarta (parte: PEstrutura; lerComEditor: boolean);
var
    nomeProg, nomeDir, nomeArq, s: string;
    arqTemp: text;
    Doc_ou_rtf: boolean;

begin
    nomeArq := pegaNomeDoArquivoDaParte (parte);
            s := trim (maiuscAnsi (parte^.content_transfer_encoding));
    if (maiuscAnsi (copy (nomeArq, length (nomeArq)-3, 4)) = '.DOC') or
       (maiuscAnsi (copy (nomeArq, length (nomeArq)-3, 4)) = '.RTF') then
        begin
            nomeArq := '$' + copy (nomeArq, length (nomeArq)-3, 4);

            if s <> 'BASE64' then
                begin
                    gravaParteDecodificada (parte, nomeArq);
                end
            else
                begin
                    gravaParteDecodificada (parte, '$.$');
                    decodMime64 ('$.$', nomeArq);
                end;
            doc_ou_rtf := TRUE;
        end
    else
        begin
            if maiuscAnsi (typeOuContent_type (parte)) = 'TEXT/HTML' then
                nomeArq := '$.html'
            else
            if maiuscAnsi (typeOuContent_type (parte)) = 'TEXT/HTM' then
                nomeArq := '$.htm';

            if s <> 'BASE64' then
            nomeArq := gravaParteDecodificada (parte, nomeArq)
            else
                begin
                    gravaParteDecodificada (parte, '$.$');
                    if trim (nomeArq) = '' then
                        nomeArq := '$2.$';
                    decodMime64 ('$.$', nomeArq);
                end;
            doc_ou_rtf := false;
        end;

    if (maiuscAnsi (typeOuContent_type (parte)) = 'TEXT/HTML') or
       (maiuscAnsi (typeOuContent_type (parte)) = 'TEXT/HTM') then
                begin
            if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
                nomeArq := dirEnvia + '\' + nomeArq
            else
                nomeArq := dirRecebe + '\' + nomeArq;
            nomeProg := sintAmbiente ('CARTAVOX', 'LEITORHTML');
        end
    else
    if lerComEditor or doc_Ou_Rtf then
        begin
            nomeProg := sintAmbiente ('CARTAVOX', 'EDITOR');
            if nomeProg = '' then
                nomeProg := 'c:\winvox\edivox.exe';
            if pos ('EDIVOX.EXE', uppercase(nomeProg)) <> 0 then
                nomeProg := '"'+ nomeProg+ '"' + ' /L';
        end
    else
        nomeProg := sintAmbiente ('CARTAVOX', 'LEITOR');

    if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
        nomeDir := dirEnvia
    else
        nomeDir := dirRecebe;
    if pos (' ', nomeDir) <> 0 then
        if nomeDir [1] <> '"' then
            nomeDir := '"' + nomeDir+ '"';

    nomeArq := trim (nomeArq);
    if pos (' ', nomeArq) <> 0 then
        if nomeArq [1] <> '"' then
            nomeArq := '"' + nomeArq + '"';
    if executaProg (nomeProg, nomeDir, nomeArq) >= 32 then
        begin
            esperaProgVoltar;
            while sintFalando do waitMessage;
        end
    else
        msgBaixo ('CTCARSUM');  {'Erro no disco: carta sumiu'}

    deletaAspas (nomeArq);
    assign (arqTemp, nomearq);
//NenoNeno
    if (maiuscAnsi (typeOuContent_type (parte)) <> 'TEXT/HTML') and
       (maiuscAnsi (typeOuContent_type (parte)) <> 'TEXT/HTM') then
        begin
            {$I-}  erase (arqTemp);  {$I+}
            if ioresult <> 0 then;
        end;
    if doc_ou_rtf then
        begin
            assign (arqTemp, '$.txt');
            {$I-}  erase (arqTemp);  {$I+}
            if ioresult <> 0 then;
        end;
    assign (arqTemp, '$.$');
    {$I-}  erase (arqTemp);  {$I+}
    if ioresult <> 0 then;
end;

{-------------------------------------------------------------}
{       leitura Mime64
{-------------------------------------------------------------}

procedure leituraMime64 (parte: pEstrutura; executa: boolean);
var
    toca: boolean;
    c, c2: char;
    extensao: string[4];
    nomeProg, nomeDir, nomeArq, s: string;
    arqNovo: file;

label erro;

begin
    nomeArq := pegaNomeDoArquivoDaParte (parte);
    if trim (nomeArq) <> '' then
        while (pos ('?', nomeArq) <> 0) and (trim (nomeArq) <> '') do
            delete (nomeArq, pos ('?', nomeArq), 1);
    toca := false;
    extensao := maiuscAnsi (copy (nomeArq, length (nomeArq)-3, 4));
    if (extensao = '.MID') or (extensao = '.WAV') or (extensao = '.MP3') then
        begin
//          se for wav, mp3 ou mid, pergunta se quer tocar
            repeat
                limpaParteTela (23, 25);
                textBackground (RED);
                mensagem ('CTTOCAR', 0);   {'Arquivo  sonoro, quer tocar (s/n) ? '}
                sintLeTecla (c, c2);
                textBackground (BLACK);
                writeln;
            until upcase (c) in ['S', 'N', ENTER, ESC];

            if upcase(c) = 'S' then
                begin
                    toca := true;
                    nomeArq := '$' + copy (nomeArq, length (nomeArq)-3, 4);
                end
            else
            if (c = ESC) or executa then
                begin
                    msgBaixo ('CTDESIST'); {'Desistiu'}
                    exit;
                end;

        end
    else
    if executa then
        begin
//          Executa a parte da carta
            repeat
                limpaParteTela (23, 25);
                textBackground (RED);
                mensagem ('CTARQVIR', 1);   {Esse arquivo pode ser um VRUS, tome muito cuidado!'}
                mensagem ('CTDESEXE', 0);   {'Deseja executar esse arquivo?'}
                sintLeTecla (c, c2);
                textBackground (BLACK);
                writeln;
            until upcase (c) in ['S', 'N', ENTER, ESC];

            if upcase(c) <> 'S' then
                begin
                    msgBaixo ('CTDESIST'); {'Desistiu'}
                    exit;
                end;
        end;

    if (not toca) and (not executa) then
        nomeArq := testaRetornaNomeArq (nomeArq, '', TypeOuContent_Type(parte));

    if trim (nomeArq) = '' then exit;

    s := trim (maiuscAnsi (parte^.content_transfer_encoding));
    msgBaixo ('CTMOMENT');  {'Um momento...'}
    if s <> 'BASE64' then
        begin
            gravaParteDecodificada (parte, nomeArq);
        end
    else
        begin
            gravaParteDecodificada (parte, '$.$');
            if not decodMime64 ('$.$', nomeArq) then goto erro;

            assignFile (arqNovo, '$.$');
            {$I-} erase (arqNovo);  {$I+}
            if ioresult <> 0 then goto ERRO;
        end;
    while sintFalando do waitMessage;

    assignFile (arqNovo, nomeArq);
    if toca then
        begin
            nomeProg := sintAmbiente ('CARTAVOX', 'TOCADOR');
            if nomeProg = '' then
                nomeProg := 'c:\winvox\midiavox.exe';
            while sintFalando do waitMessage;

            if tipoFolheGlobal in ['P', 'T', ^P , ^T] then
                nomeDir := dirEnvia
            else
                nomeDir := dirRecebe;

            nomeArq := trim (nomeArq);
            if pos (' ', nomeArq) <> 0 then
                if nomeArq [1] <> '"' then
                    nomeArq := '"' + nomeArq + '"';
            if executaProg (nomeProg, nomeDir, nomeArq) >= 32 then
                esperaProgVoltar;
            {$I-} erase (arqNovo);  {$I+}
            if ioresult <> 0 then;
        end
    else
    if (extensao = '') and executa then
        begin
            LeituraParteCarta (parte, TRUE);
            {$I-} erase (arqNovo);  {$I+}
            if ioresult <> 0 then;
        end
    else
    if executa then
        begin
            if (extensao = 'EXE') or (extensao = 'COM') then
                begin
                    if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
                        nomeProg := dirEnvia + '\'+nomeArq
                    else
                        nomeProg := dirRecebe+'\'+nomeArq;
                    nomeArq := '';
                end
            else
                nomeProg := sintAmbiente ('DOSVOX', 'PROG' + extensao);
            if nomeProg = '' then
                begin
                    if tipoFolheGlobal in ['P', 'T', ^P, ^T] then
                        nomeDir := dirEnvia
                    else
                        nomeDir := dirRecebe;
                end;

            while sintFalando do waitMessage;
            nomeArq := trim (nomeArq);
            if pos (' ', nomeArq) <> 0 then
                if nomeArq [1] <> '"' then
                    nomeArq := '"' + nomeArq + '"';
            if nomeProg = '' then
                begin
                    nomeProg := nomeArq;
                    nomeArq := '';
                end;
            if executaProg (nomeProg, nomeDir, nomeArq) >= 32 then
                esperaProgVoltar;
            while sintFalando do waitMessage;

            {$I-} erase (arqNovo);  {$I+}
            if ioresult <> 0 then;
        end
    else
        msgBaixo ('CTOK');    {'OK'}
    exit;

erro:
    msgBaixo ('CTERRDSK');    {'Erro de escrita no disco'}
    if ioresult <> 0 then;
end;

{-------------------------------------------------------------}
{       Grava a(s) parte(s) em disco, pede o diretrio.
{-------------------------------------------------------------}


procedure gravaParteDisco (nParte: integer);
var
    diretorio, nomeArq, item: string;
    arqTemp: text;
    i: integer;
    c, c2: char;
    gravarSelecionadas: boolean;
    parte: PEstrutura;
label ERRO, desistiu, semDiretorio;

begin
    limpaParteTela (20, 25);
    diretorio := dirAnexo;
    if not selecDirDest (diretorio) then
        goto semDiretorio;

    diretorio := trim (diretorio);
    if diretorio [length (diretorio)] in ['\','/'] then delete (diretorio, length (diretorio), 1);

    if temItemSelecionado then
    begin
            repeat
                limpaParteTela (23, 25);
                textBackground (RED);
                mensagem ('CTGRASEL', 0); {'Deseja gravar as partes selecionadas?'}
                sintLeTecla (c, c2);
                textBackground (BLACK);
                writeln;
            until upcase (c) in ['S', 'N', ESC, ENTER];
            if c = ESC then goto desistiu;
            if upcase (c) in ['S', ENTER] then
                gravarSelecionadas := TRUE
            else
                gravarSelecionadas := false;
        end
    else
        gravarSelecionadas := false;

    if gravarSelecionadas then
        begin
            msgBaixo ('CTMOMENT');  {'Um momento...'}
            for i := 1 to folheiaNumItens do
                begin
                    folheiaObtemItem (i, item, gravarSelecionadas);
                    if not gravarSelecionadas then continue;
                    parte := listaDePartes [strToInt (trim (copy (item, 1, 5))) - 1];
                    nomeArq := pegaNomeDoArquivoDaParte (parte);
                    if trim (nomeArq) <> '' then
                        while (pos ('?', nomeArq) <> 0) and (trim (nomeArq) <> '') do
                            delete (nomeArq, pos ('?', nomeArq), 1);

                    nomeArq := testaRetornaNomeArq (nomeArq, diretorio, TypeOuContent_Type (parte));
                    if trim (nomeArq) = '' then
                        begin
                            repeat
                                limpaParteTela (24, 25);
                                textBackground (RED);
                                mensagem ('CTCGRAPS', 0); {'Deseja continuar a gravao das partes selecionadas?'}
                                sintLeTecla (c, c2);
                                textBackground (BLACK);
                                writeln;
                            until upcase (c) in ['S', 'N', ENTER, ESC];
                            if upcase (c) in ['N', ESC] then
                                goto desistiu
                            else
                                msgBaixo ('CTOK'); {'OK'}
                        end
                    else
                        begin
                            limpaParteTela (20, 25);
                            textBackground (RED);
                            mensagem ('CTGRAVPA', 0); {'Gravando parte '}
//                            sintWriteInt (i);
                            sintwrite (parte^.numero);
                            textBackground (BLACK);
                            writeln;

                            item := trim (maiuscAnsi (parte^.content_transfer_encoding));
                            if item <> 'BASE64' then
                                begin
                                    gravaParteDecodificada (parte, nomeArq);
                                end
                            else
                                begin
                                    gravaParteDecodificada (parte, '$.$');
                                    if not decodMime64 ('$.$', nomeArq) then
                                        goto erro;

                                    assignFile (arqTemp, '$.$');
                                    {$I-} erase (arqTemp);  {$I+}
                                    if ioresult <> 0 then
                                        goto ERRO;
                                end;
                            while sintFalando do waitMessage;
                        end;
                end;
        end
    else
        begin
            parte := listaDePartes [nParte];
            nomeArq := pegaNomeDoArquivoDaParte (parte);
            if trim (nomeArq) <> '' then
                while (pos ('?', nomeArq) <> 0) and (trim (nomeArq) <> '') do
                    delete (nomeArq, pos ('?', nomeArq), 1);
            nomeArq := testaRetornaNomeArq (nomeArq, diretorio, TypeOuContent_Type (parte));
            msgBaixo ('CTMOMENT');  {'Um momento...'}
            if trim (nomeArq) = '' then exit;

            item := trim (maiuscAnsi (parte^.content_transfer_encoding));
            if item <> 'BASE64' then
                begin
                    gravaParteDecodificada (parte, nomeArq);
                end
            else
                begin
                    gravaParteDecodificada (parte, '$.$');
                    if not decodMime64 ('$.$', nomeArq) then
                        goto erro;
                    assignFile (arqTemp, '$.$');
                    {$I-} erase (arqTemp);  {$I+}
                    if ioresult <> 0 then
                        goto ERRO;
                end;
            while sintFalando do waitMessage;
        end;

    msgBaixo ('CTOK'); {'OK'}
    exit;
desistiu:
    msgBaixo ('CTDESIST'); {'Desistiu'}
    exit;
ERRO:
    msgBaixo ('CTERRDSK');    {'Erro de escrita no disco'}
    exit;
semDiretorio:
end;

{-------------------------------------------------------------}
{       Executa ou grava a parte
{-------------------------------------------------------------}

procedure leOuGravaParte (parte: pEstrutura; lerSempre:boolean);
var
    tipo: string;
begin
    tipo := trim (maiuscAnsi (typeOuContent_type (parte)));
    if (tipo = 'TEXT/HTML') or (tipo = 'TEXT/PLAIN') or
       ((parte^.carta <> nil) and (tipo = '')) or lerSempre then
        LeituraParteCarta (parte, false)
    else
        leituraMime64 (parte, false);

end;

{--------------------------------------------------------}
{       seleciona a funo da parte da carta com as setas
{--------------------------------------------------------}

function selSetasFolheiaPartes (Var c: char): char;
var n: integer;
const
    tabLetrasFolheia: string [16] = 'LEG'+ ESQ + DIR +'AQ'+ ^Q +'S */P'+ ^F + ESC + F12;

begin
    c := #0;
    popupMenuCria (35, wherey, 50, 15, RED);
    MenuAdiciona ('CTAJFP02');  {'ENTER ou L - Ler ou gravar a parte'}
    MenuAdiciona ('CTAJFP03'); {'E - Ler a parte com o editor'}
    MenuAdiciona ('CTAJFP04'); {'G - Gravar a parte'}
    MenuAdiciona ('CTAJFP05'); {'SETA PARA ESQUERDA - Informa nome do arquivo da parte'}
    MenuAdiciona ('CTAJFP06'); {'SETA PARA DIREITA - Informa tipo da parte'}
    MenuAdiciona ('CTAJFP07'); {'A - Apagar a parte'}
    MenuAdiciona ('CTAJFP08'); {'Q - Informa quantas do total'}
    MenuAdiciona ('CTAJFP09'); {'CTRL+Q - Informa quantas selecionadas do total'}
    MenuAdiciona ('CTAJFP10'); {'S - Informa assunto da carta'}
    MenuAdiciona ('CTAJFP11'); {'ESPAO - Seleciona ou tira seleo'}
    MenuAdiciona ('CTAJFP12'); {'* - Seleciona tudo'}
    MenuAdiciona ('CTAJFP13'); {'/ - Tira seleo de tudo'}
    MenuAdiciona ('CTAJFP14'); {'P - Lista cabealho da parte'}
    MenuAdiciona ('CTAJFP15'); {'CTRL+F - Grava carta e sai'}
    MenuAdiciona ('CTAJFP17'); {'ESC - Terminar folheamento das partes'}

    n := popupMenuSeleciona;
    if (n <= 0) or (n > 15) then
        n := 16;
    if  not (n in [4, 5, 16]) then
        selSetasFolheiaPartes := tabLetrasFolheia[n]
    else
        begin
            c := tabLetrasFolheia[n];
            selSetasFolheiaPartes := #0;
        end;

end;

{-------------------------------------------------------------}
{       Ajuda do folheamento das partes da carta
{-------------------------------------------------------------}

procedure ajudaFolhearPartes;
begin
    limpaParteTela (6, 25);
    textBackground(BLUE);
    if not keypressed then
        mensagem ('CTAJFP01', 2);  {'Folheie as partes com as setas, depois tecle:'}
    if not keypressed then
        mensagem ('CTAJFP02', 1);  {'ENTER ou L - Ler ou gravar a parte'}
    if not keypressed then
        mensagem ('CTAJFP03', 1); {'E - Ler a parte com o editor'}
    if not keypressed then
        mensagem ('CTAJFP04', 1); {'G - Gravar a parte'}
    if not keypressed then
        mensagem ('CTAJFP05', 1); {'SETA PARA ESQUERDA - Informa nome do arquivo da parte'}
    if not keypressed then
        mensagem ('CTAJFP06', 1); {'SETA PARA DIREITA - Informa tipo da parte'}
    if not keypressed then
        mensagem ('CTAJFP07', 2); {'A - Apagar a parte'}

    if not keypressed then
        mensagem ('CTATCLF9', 0); {'Tecle F9 para selecionar a opo com as setas'}
    textBackground(BLACK);  clreol;
    writeln;
    readkey;
    if keypressed then readkey;
end;

{-------------------------------------------------------------}
{       inicializa o folheamento das partes da carta
{-------------------------------------------------------------}

procedure inicializaFolheamento;
var
    totalPartes, i: integer;
    tipo, nomeArq: string;
    parte: PEstrutura;
begin
    totalPartes := listaDePartes.count;

    folheiaCria (1, 6, 80, 17);
    for i :=  0 to (totalPartes - 1) do
        begin
            parte := listaDePartes[i];
            if not parte^.removido then
                begin
                    tipo := typeOuContent_type (parte);
                    if trim (tipo) = '' then tipo := pegaTextoMensagem ('CTNAODEF'); {'No definido'}
                    nomeArq := pegaNomeDoArquivoDaParte (parte);
                    nomeArq := copy (nomeArq + BRANCOS, 1, 54);
                    folheiaAdicionaEspecial (copy (intToStr(i+1)+BRANCOS, 1, 5) +
                                    copy (parte^.numero+BRANCOS, 1, 10) +
                                    copy (tipo+BRANCOS, 1, 20)+
                                    copy (nomeArq+BRANCOS, 1, 39),
                                    parte^.selecionado, parte^.numero+
                                    ' '+nomeArq+' '+tipo);
                end;
        end;
    totalPartesFolhe := folheiaNumItens;
end;

{-------------------------------------------------------------}
{                     folheamento das partes
{-------------------------------------------------------------}

procedure folheamentoDasPartes (ncar: integer);
var
    c, c2: char;
    nParte, nFolhe: integer;
    item: string;
    selecionado: boolean;
    parte: PEstrutura;
    primeiraVez, podeFalar: boolean;

begin
    inicializaFolheamento;

    primeiraVez := true;
    nFolhe := 1;
    podeFalar := true;
    repeat
        telaFolheamentoPartes (ncar, folheiaNumItens);
        if primeiraVez then
            begin
                mensagem ('CTNARQM', -1); {'Nmero de partes inclusas: '}
                sintetiza (intToStr (folheiaNumItens));
                mensagem ('CTQPARTE', -1); {'Qual a parte desejada? (use as setas, f1 ajuda)'}
                primeiraVez := false;
            end;
        folheiaCorDoMeio (1, 35, CYAN);
        folheiaExecuta (nFolhe, nFolhe, c, c2, podeFalar);

        if nFolhe < 1 then NFolhe := 1;
        if nfolhe > folheiaNumItens then nFolhe := folheiaNumItens;
        for nParte := 1 to folheiaNumItens do
            begin
                folheiaObtemItem (nParte, item, selecionado);
                parte := listaDePartes[strToInt (trim (copy (item, 1, 5))) - 1];
                parte^.selecionado := selecionado;
            end;
        folheiaObtemItem (nFolhe, item, selecionado);
        nParte := strToInt (trim (copy (item, 1, 5))) - 1;
        sintPara;
        parte := listaDePartes[nParte];

        if c2 = F9 then
            c := selSetasFolheiaPartes (c2);

        if c = #0 then
            begin
                case c2 of
                    ESQ:    falaNomeDoArquivoDaParte (nParte);
                    DIR:    falaTipoDaParte (nParte);
                    F1:     ajudaFolhearPartes;
                    F7:     apagaParte (nFolhe);
                    F8:     falaHora;
                    CTLF8:  falaDia;
                end;
        end
        else
            begin
                    case upcase(c) of
                        ENTER, 'L': leOuGravaParte
(listaDePartes[nParte], false);
                        'E': LeituraParteCarta (parte, TRUE);
                        'G': gravaParteDisco (nParte);
                        'A': apagaParte (nFolhe);
                        'S': falaAssunto (regLido[nCar]^.carta^.subject);
                        'Q': falaQualItemDeQuantos (nFolhe, false);
                        ^Q: falaQualItemDeQuantos (nFolhe, TRUE);
                        'X': leituraMime64 (listaDePartes [nParte], true);
                        ' ':selecionaOuNaoUmItem (nFolhe);
                        '+': folheiaSeleciona (nFolhe, true);
                        '-': folheiaSeleciona (nFolhe, false);
                        '*': selecionaTodosItens (TRUE);
                        '/': selecionaTodosItens (false);

                        'P':
                            begin
                                folheiaDestroi;
                                if parte^.carta = NIL then
                                    mostraItensParte (parte)
                                else
                                    mostraItensCarta (parte);
                                inicializaFolheamento;
                            end;

                        ^F: //Grava a carta e sai do folheamento
                            begin
                                gravacaoForcada := true;
                                c := ESC;
                            end;

                    else
                        if c <> ESC then msgBaixo ('CTOPVINV'); {'Opo invlida, aperte F1 para ajuda'}
                    end;
            end;

        if folheiaNumItens < 1 then
            msgBaixo ('CTDESIST') {'Desistiu'}
        else
        if upcase(c) in [ENTER, 'L', 'E', 'G', 'A', 'P'] then
            mensagem ('CTQPARTE', -1); {'Qual a parte desejada? (use as setas, f1 ajuda)'}

        if (upCase(c) in['S', 'Q', ^Q]) or (c2 in [ESQ, DIR, f8, CTLF8]) then
            podeFalar := false
        else
            podeFalar := true;

    until (c = ESC) or (folheiaNumItens < 1);

    folheiaDestroi;
end;

{-------------------------------------------------------------}
{       Escolhe o tipo de leitura, carta simples ou multipart
{-------------------------------------------------------------}

procedure leCarta (ncar: integer; lerSempre: boolean);
var
 nomeArq, s, sData, sHora, dataAux : string;
 arqAux : textfile;
 r1 : char;
 pCc : PListaCC;
 Year, Month, Day, DayOfWeek: Word;
 Hour, Minute, Second, Sec100: Word;
begin
    gravacaoForcada := false;
    if not carregaLinhasArquivo (regLido[ncar]^.carta^.nomArqCarta) then//Aloca o arquivo da carta
        begin
            msgBaixo ('CTCARSUM');  {'Erro no disco: carta sumiu'}
            exit;
        end;

    if linhasArquivo.count > 20000 then
        msgBaixo ('CTMOMENT');   {'Um momento...  '}
    preencheCartaPrincipal (regLido[ncar]);//preenche o cabealho principal da carta
    if linhasArquivo.count > 20000 then
        msgBaixo (''); {''}

    //verifica se pede confirmao de recebimento e se a carta ainda no foi lida
    if (regLido[ncar]^.carta^.disposition_notification_to <> '') and
       (pos ('M.', regLido[nCar]^.carta^.nomArqCarta) = 0)  then
    begin
        repeat
                mensagem('CTRESCON',0); {'O remetente pediu confirmao de leitura. Posso confirmar?'}
                r1 := sintReadKey();
                if upcase(r1) = 'S' then
                begin
                    getDate(Year, Month, Day, DayOfWeek);
                    str (Day, sData);
                    if length(sData) < 2 then sData := '0' + sData;
                    sData := sData + '/';
                    str (Month, s);
                    if length(s) < 2 then s := '0' + s;
                    sData := sData + s;
                    str(Year, s);
                    sData := sData + '/' + s;

                    dvcrt.getTime (Hour, Minute, Second, Sec100);
                    str(Hour, sHora);
                    if length(sHora) < 2 then sHora := '0' + sHora;
                    sHora := sHora + ':';
                    str(Minute, s);
                    if length(s) < 2 then s := '0' + s;
                    sHora := sHora + s;

                    dataAux := regLido[ncar]^.carta^.date;
                    if pos(',', dataAux) > 0 then
                        dataAux := copy(dataAux, pos(',', dataAux) + 2, length(dataAux));
                    if pos('-0', dataAux) > 0 then
                        dataAux := copy(dataAux, 1, pos('-0', dataAux) - 1);

                    nomeArq := dirRecebe + '\' + 'Conf_receb' + IntToStr(Year) +
                               IntToStr(Month)  + IntToStr(Second) + IntToStr(Hour) +
                               IntToStr(Minute) + IntToStr(Second) + '.txt';
                    assignFile(arqAux, nomeArq);
                    {$I-} rewrite(arqAux); {$I+}
                    if ioresult = 0 then
                         begin
                            writeln(arqAux, 'Esta  uma confirmao de leitura do email que voc enviou para ' +
                                    regLido[ncar]^.carta^.to_ + ' no dia ' + dataAux);
                            writeln(arqAux, ' Esta confirmao informa que a carta foi aberta pelo destinatrio dia ' +
                                    sData + ' as ' + sHora);
                            {$I-} closeFile(arqAux); {$I+}
                            if ioresult <> 0 then;
                            pcc := nil;
                            geraTextoSmtp(nomeArq, regLido[nCar]^.carta^.from, 'Confirmao de Recebimento: ' +
                                    regLido[ncar]^.carta^.subject, pCc, pcc, false, false, false, 0, '');
                         end;
                end;
        until upcase (r1) in ['S', 'N', ESC];
    end;

    if trim (regLido[ncar]^.boundary) = '' then  //No tem anexos
        leOuGravaParte (regLido [nCar], lerSempre)
//        LeituraParteCarta (regLido[ncar], false)
    else  //Se tiver anexos
        begin
            criaIndicePartes;   //Cria o TList das partes que  preenchido no montatudo
            montaTudo (regLido[ncar]);   //Cria a estrutura da carta na memria
            if listaDePartes.count = 1 then  //Apenas uma parte
                leOuGravaParte (listaDePartes[0], false)
            else  //Mais de uma parte
                begin
                    folheamentoDasPartes (ncar);//Exibe a lista das partes com opes
                    gravarAlteracoes (ncar);
                end;
            desmontaTudo (regLido[ncar]); //Desaloca a estrutura da carta
            destroiIndicePartes; //Destroi o TList com o indice das partes.
        end;

    destroiLinhasArquivo;//Desaloca o arquivo da carta
end;

{-------------------------------------------------------------}
begin
end.
